#define _GNU_SOURCE

#include <libguile.h>

#include <string.h>

#include <libintl.h>
#include <locale.h>
#include <errno.h>

#if  (SCM_MAJOR_VERSION  == 1) && (SCM_MINOR_VERSION == 6)

SCM scm_from_locale_string(const char *str)
{
	return scm_mem2string(str?str:"",str?strlen(str):0);
}

int scm_is_string(SCM str)
{
	return SCM_STRINGP(str);
}

char *scm_to_locale_string(SCM str)
{
	return strdup(SCM_STRING_CHARS(str));
}
#endif

SCM
scm_dlgettext(SCM domain, SCM msgid, SCM locale)
{
    SCM_ASSERT(scm_is_string(domain), domain, SCM_ARG1, "dlgettext");
    SCM_ASSERT(scm_is_string(msgid), msgid, SCM_ARG1, "dlgettext");
    SCM_ASSERT(scm_is_string(locale), locale, SCM_ARG1, "dlgettext");

    char *s_domain = scm_to_locale_string(domain);
    char *s_msgid = scm_to_locale_string(msgid);
    char *s_locale = scm_to_locale_string(locale);

    locale_t	new_locale = NULL;
    locale_t	old_locale = NULL;
    const char *s_result = s_msgid;
    SCM  result;

    if ((new_locale = newlocale(LC_CTYPE_MASK|LC_MESSAGES_MASK, s_locale, NULL)) != NULL)
    {
	old_locale = uselocale(new_locale);
	s_result = dgettext(s_domain,s_msgid);
	uselocale(old_locale);

	freelocale(new_locale);
    }

    result = scm_from_locale_string(s_result);

    free(s_domain);
    free(s_msgid);
    free(s_locale);

    return result;
}

void scm_init_gettext(void)
{
    scm_c_define_gsubr("dlgettext", 3, 0, 0, scm_dlgettext);
}
